# import DB
# from UNKnownDB.UNDL import Interpreter, APP
from UNKnownDB.DB import LightDB

with LightDB.Data('./light') as db:
    db + 'ed:de'
    db - 'ed:de'
    db['fe'] = 'ef0'
    print(db['fe'])
    print(db.all())
with open('./light.unl') as light:
    print(light.read())
# db = DB.LocalDB("./.Clever.unp")
# db.create()



"""
Light DataBase
"""
import os
import re


class Data:
    def __init__(self, path, name='light'):
        self.path = path + '.unl'
        self.name = name

    def __enter__(self, ):
        if not os.path.isfile(self.path):
            open_ = open(self.path, 'wb')
            open_.write(bytes('' + self.name + '', 'UTF-8'))
            open_.close()
        self.DB = open(self.path, 'rb+')
        self.read = self.DB.read()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.DB.close()
        with open(self.path, 'r') as db:
            dbb = db.read()
        with open(self.path, 'w') as new:
            new.write(dbb.replace(chr(0), ''))

    def __add__(self, other):
        """+"""
        write = re.findall('(.+):(.+)', other)
        self.DB.seek(0, 2)
        self.DB.write(bytes('' + write[0][0] + '' + write[0][1] + '', 'utf-8'))
        return self

    def __sub__(self, other):
        """-"""
        self.DB.seek(0, 0)
        changing = str(self.DB.read(), 'utf-8')
        write = re.findall('(.+):(.+)', other)
        change = '' + write[0][0] + '' + write[0][1] + ''
        change_list = [(m.group(), m.span()) for m in re.finditer(change, changing)]
        self.DB.seek(change_list[0][1][0], 0)
        self.DB.truncate(change_list[0][1][0])
        self.DB.seek(0, 2)
        self.DB.write(bytes(changing[change_list[0][1][1]:], 'utf-8'))
        return self

    def __setitem__(self, key, value):
        """[] = ?"""
        self.DB.seek(0, 0)
        if '' + key + '' in str(self.read, 'utf-8'):
            change_list = [(m.group(), m.span()) for m in re.finditer('' + key + '.+?', str(self.DB.read(), 'utf-8'))]
            self.DB.seek(change_list[0][1][0], 0)
            self.DB.truncate(change_list[0][1][0])
            self.DB.seek(0, 2)
            self.DB.write(bytes('' + key + '' + value + '', 'utf-8'))
        else:
            self.DB.seek(0, 2)
            self.DB.write(bytes('' + key+ '' + value + '', 'utf-8'))

    def __getitem__(self, item):
        """[]"""
        self.DB.seek(0, 0)
        returns =  [(m.group(), m.span()) for m in re.finditer('' + item + '.+?', str(self.DB.read(), 'utf-8'))]
        return [re.sub('|||' + item, '', i[0]) for i in returns]

    def all(self):
        for m in re.finditer('.+?.+?', str(self.read, 'utf-8')):
            return re.sub('|||', '',  m.group(),), m.span()

    def find(self):
        pass


class File(Data):
    def __add__(self, other):
        """+"""
        write = re.findall('(.+):(.+)', other)
        self.DB.seek(0, 2)
        if os.path.isfile(write[0][1]) is True:
            split = os.path.split(write[0][1])
            os.rename(write[0][1], str(hash(split[0])) + '.unf' + split[1])
            self.DB.write(bytes('' + write[0][0] + '' + os.path.abspath(write[0][1]) + '', 'utf-8'))
        else:
            return False
        return self



"""A DB"""
import os
import shutil
import sqlite3
import UNKnownDB.UNDL.Interpreter


class LocalDB:
    def __init__(self, path):
        self.path = path
        self.db = None
        self.dbData = None

    def create(self):
        base_path = ['.OBJECT.unp', '.MAPPING.unp']
        try:
            os.mkdir(self.path)
            os.chdir(self.path)
            for i in base_path:
                os.mkdir('./' + i)
            os.mknod('./Guide.undl')
            with open('./Guide.undl', 'w') as self.dbData:
                split = os.path.split(self.path)
                split_text = os.path.splitext(split[1])
                self.dbData.write(
                    'Guide:\n'
                    '    Name:' + split_text[0].removesuffix('.') + '\n'
                    '    IP: <None>\n'
                    '    Port:4466 \n'
                    'Form:try\n'
                    '    Head:clever creator is clever\n'
                    '    Body:op po oo pp\n'
                                  )
        except FileExistsError:
            self.delete_all(self.path)
            os.rmdir(self.path)

    def __enter__(self):
        return UNKnownDB.UNDL.Interpreter

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.sql = None
        self.sqlData = None

    def delete_all(self, delete_path):
        if not os.listdir(delete_path):
            pass
        else:
            for i in os.listdir(delete_path):
                path_file = os.path.join(delete_path, i)
                if os.path.isfile(path_file):
                    os.remove(path_file)
                else:
                    self.delete_all(path_file)
                    shutil.rmtree(path_file)

    def write(self):
        pass


class WebDB:
    def __init__(self):
        self.form = None


class SQLITE:
    def __init__(self, path, sqlite, table):
        self.connection = sqlite3.connect(sqlite)
        self.cursor = self.connection.cursor()
        self.db = UNKnownDB.UNDL.Interpreter.Interpret(path)
        self.cursor.execute('create table ' + table + self.db.Form[0])
        self.cursor.close()
        self.connection.commit()



import sys
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QApplication, QWidget, QLabel, QListWidget, QListWidgetItem, QHBoxLayout


class Demo(QWidget):
    def __init__(self):
        super(Demo, self).__init__()
        self.pic_label = QLabel(self)
        self.pic_label.setPixmap(QPixmap('arrow.png'))

        self.listwidget_1 = QListWidget(self)  #实例化列表控件
        self.listwidget_2 = QListWidget(self)
        self.listwidget_1.doubleClicked.connect(lambda: self.change_func(self.listwidget_1))
        #双击列表控件时发出信号
        self.listwidget_2.doubleClicked.connect(lambda: self.change_func(self.listwidget_2))

        for i in range(6):
            text = 'Item {}'.format(i)
            self.item = QListWidgetItem(text)  #把字符串转化为QListWidgetItem项目对象
            self.listwidget_1.addItem(self.item)  #添加项目

        self.item_6 = QListWidgetItem('Item 6', self.listwidget_1)  # 实例化后直接添加

        self.listwidget_1.addItem('Item 7')  #直接添加项目，不用QListWidgetItem对象，【功能可能不全】
        str_list = ['Item 9', 'Item 10']
        self.listwidget_1.addItems(str_list)  #添加项目-列表

        self.item_8 = QListWidgetItem('Item 8')
        self.listwidget_1.insertItem(8, self.item_8)  #插入项目。参数1：索引号,参数2：项目
        # self.listwidget_1.insertItem(8, 'Item 8')

        self.h_layout = QHBoxLayout()
        self.h_layout.addWidget(self.listwidget_1)
        self.h_layout.addWidget(self.pic_label)
        self.h_layout.addWidget(self.listwidget_2)
        self.setLayout(self.h_layout)

        self.listwidget_1.itemClicked.connect(self.d)  #单击列表控件时发出信号
        self.listwidget_1.currentItemChanged.connect(self.g)  #当前项目发生变化时发出信号
        self.listwidget_1.addItem('Item_11')

    def g(self):
        print('项目总数发生了改变')

    def d(self):
        print('你单击了列表控件')

    def change_func(self, listwidget):
        if listwidget == self.listwidget_1:
            item = QListWidgetItem(self.listwidget_1.currentItem())  #转化为QListWidgetItem对象
            # self.listwidget_1.currentItem()   返回当前项目。是个对象。<PyQt5.QtWidgets.QListWidgetItem object at 0x0000008425463A68>
            self.listwidget_2.addItem(item)  #添加项目。参数是QListWidgetItem对象
            print(self.listwidget_2.count()) #返回项目总数

        else:
            self.listwidget_2.takeItem(self.listwidget_2.currentRow())#删除指定索引号的项目
            #self.listwidget_2.currentRow()    返回当前项目的行索引号
            print(self.listwidget_2.count())


if __name__ == '__main__':
    app = QApplication(sys.argv)
    demo = Demo()
    demo.show()
    sys.exit(app.exec_())



import sys


from PyQt5.Qt import *


on_file = ''
in_file = ''
open_file = ''
close_file = ''


class Tree(QWidget):

    def __init__(self):
        super().__init__()
        self.title = 'UNDL IDE'
        self.left = 0
        self.top = 40
        self.width = 1900
        self.height = 1020
        self.init_ui()
        self.model = None
        self.tree = None
        self.app = None
        self.setLayout = None
        self.TableWidget = None
        self.layout = None

    def init_ui(self):
        self.setWindowTitle(self.title)
        self.setGeometry(self.left, self.top, self.width, self.height)
        self.model = QFileSystemModel()
        self.model.setRootPath('')
        self.tree = QTreeView()
        self.tree.setModel(self.model)
        self.tree.setAnimated(False)
        for i in [1, 2, 3]:
            self.tree.setColumnHidden(i, True)
        self.tree.setIndentation(10)
        self.tree.setSortingEnabled(True)
        self.tree.setWindowTitle("Event")
        self.tree.resize(400, 1000)
        window_layout = QVBoxLayout()
        window_layout.addWidget(self.tree)
        self.setLayout(window_layout)
        self.show()


class SQLConnect(QWidget):
    def __init__(self):
        super(SQLConnect, self).__init__()
        self.name = QTextEdit()
        window_layout = QVBoxLayout()
        window_layout.addWidget(self.name)
        self.setLayout(window_layout)
        self.show()


class File(QWidget):
    def __init__(self):
        super(File, self).__init__()
        self.textEdit = QTextEdit()
        self.textEdit.setGeometry(100, 100, 100, 30)
        self.textEdit.setPlaceholderText('UNDL Code')
        self.textEdit.setAutoFormatting(QTextEdit.AutoBulletList)
        self.textEdit.setAutoFormatting(QTextEdit.AutoAll)
        self.textEdit.setFontFamily('Quicksand')
        self.textEdit.setFontWeight(QFont.Bold)
        self.textEdit.setFontPointSize(15)
        self.textEdit.setTextBackgroundColor(QColor(250, 250, 250))
        global on_file
        on_file = self.textEdit.toPlainText()

        # 创建一个布局管理器
        self.setWindowTitle('File Edit')
        window_layout = QVBoxLayout()
        window_layout.addWidget(self.textEdit)
        self.setLayout(window_layout)
        self.show()
        
        
class Table(QWidget):
    def __init__(self):
        super(Table, self).__init__()


class Left(QWidget):
    def __init__(self):
        super(Left, self).__init__()
        self.block = QSplitter(Qt.Vertical)
        self.block.addWidget(Tree())
        self.block.addWidget(SQLConnect())
        self.block.setSizes([200, 50])
        window_layout = QVBoxLayout()
        window_layout.addWidget(self.block)
        self.setLayout(window_layout)
        self.show()


class Middle(QWidget):
    def __init__(self):
        super(Middle, self).__init__()
        self.tab = QTabWidget()
        self.tab.addTab(File(), 'main')
        window_layout = QVBoxLayout()
        window_layout.addWidget(self.tab)
        self.setLayout(window_layout)


class SPL(QSplitter):
    def __init__(self):
        super(SPL, self).__init__()
        self.addWidget(Left())
        self.addWidget(Middle())
        self.setSizes([150, 500])


class MainWindow(QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()
        self.pressed = None
        self.setCentralWidget(SPL())
        self.resize(1910, 1010)
        self.status = self.statusBar()
        self.status.showMessage('Nothing')
        # 菜单栏
        menu_bar = self.menuBar()
        menu = menu_bar.addMenu("File")
        action = QAction(QIcon("./Image/run.ico"), "New Project", self)
        menu.addAction(action)
        menu2 = menu.addMenu("Add to ...")
        menu2.addAction(QAction("workspace edit...", self))
        # 工具栏
        tool = self.addToolBar("File")
        run = QAction(QIcon("/home/pi/PycharmProjects/UNKnownDB/UNKnownDB/UNDL/APP/Image/run.ico"), "run", self)
        tool.addAction(run)
        tool.actionTriggered[QAction].connect(self.tool_btn_pressed)
        #
        self.status = QStatusBar()
        self.setStatusBar(self.status)
        #
        self.setWindowState(Qt.WindowMaximized)
        self.show()

    def tool_btn_pressed(self, q_action):
        self.pressed = str("pressed too btn is " + q_action.text())
        if q_action == 'run':
            pass
        self.status.showMessage(self.pressed)


if __name__ != '__main__':
    app = QApplication(sys.argv)
    app.setWindowIcon(QIcon(''))
    window = MainWindow()
    sys.exit(app.exec_())



from setuptools import setup, find_packages


setup(
    name='UNKnownDB',
    version='1.6.1',
    author='CleaverCreator',
    author_email='liuhanbo333@icloud.com',
    packages=find_packages(),
    zip_safe=False,
    platforms=['Linux'],
    install_requires=['PyQt5'],
    description='A new Python DB',
    long_description="""
                    A DB that human can read
                    'Use Unknown Describe Language
                                    """,
    license='GIL',
    url='https://github.com/CleverCreater/UNKnownDB',
    classifiers=[]
)
